package ch.brotzilla.performance.settings;

import java.util.Properties;

public class PerfTestSettings extends AbstractSettings {
    
    public static final String 
        Key_RandomSeed = "randomSeed",
        Key_Width = "width", Key_Height = "height",
        Key_NumberOfThreads = "numberOfThreads", Key_NumberOfIterations = "numberOfIterations", Key_NumberOfTriangles = "numberOfTriangles",
        Key_UseTransparency = "useTransparency", Key_UseAntialiasing = "useAntialiasing";

    private String randomSeed;
    private int width, height, numberOfThreads, numberOfIterations, numberOfTriangles;
    private boolean useTransparency, useAntialiasing;
    
    public PerfTestSettings() {
        super();
    }
    
    public PerfTestSettings(String prefix, Properties properties) {
        super(prefix, properties);
    }
    
    public String getRandomSeed() {
        return randomSeed;
    }
    
    public void setRandomSeed(String randomSeed) {
        this.randomSeed = randomSeed;
    }
    
    public int getWidth() {
        return width;
    }
    
    public void setWidth(int width) {
        this.width = width;
    }
    
    public int getHeight() {
        return height;
    }
    
    public void setHeight(int height) {
        this.height = height;
    }
    
    public int getSize() {
        return width * height;
    }
    
    public int getNumberOfThreads() {
        return numberOfThreads;
    }
    
    public void setNumberOfThreads(int numberOfThreads) {
        this.numberOfThreads = numberOfThreads;
    }
    
    public int getNumberOfIterations() {
        return numberOfIterations;
    }
    
    public void setNumberOfIterations(int numberOfIterations) {
        this.numberOfIterations = numberOfIterations;
    }
    
    public int getNumberOfTriangles() {
        return numberOfTriangles;
    }
    
    public void setNumberOfTriangles(int numberOfTriangles) {
        this.numberOfTriangles = numberOfTriangles;
    }
    
    public boolean isUseTransparency() {
        return useTransparency;
    }
    
    public void setUseTransparency(boolean useTransparency) {
        this.useTransparency = useTransparency;
    }
    
    public boolean isUseAntialiasing() {
        return useAntialiasing;
    }
    
    public void setUseAntialiasing(boolean useAntialiasing) {
        this.useAntialiasing = useAntialiasing;
    }
    
    @Override
    public void validate() {
        checkNotEmpty(randomSeed, Key_RandomSeed);
        checkGreaterThanZero(width, Key_Width);
        checkGreaterThanZero(height, Key_Height);
        checkGreaterThanZero(numberOfThreads, Key_NumberOfThreads);
        checkGreaterThanZero(numberOfIterations, Key_NumberOfIterations);
        checkGreaterThanZero(numberOfTriangles, Key_NumberOfTriangles);
    }

    @Override
    protected void _loadProperties(String prefix, Properties properties) {
        randomSeed = loadString(prefix, Key_RandomSeed, properties, randomSeed);
        width = loadInteger(prefix, Key_Width, properties, width);
        height = loadInteger(prefix, Key_Height, properties, height);
        numberOfThreads = loadInteger(prefix, Key_NumberOfThreads, properties, numberOfThreads);
        numberOfIterations = loadInteger(prefix, Key_NumberOfIterations, properties, numberOfIterations);
        numberOfTriangles = loadInteger(prefix, Key_NumberOfTriangles, properties, numberOfTriangles);
        useTransparency = loadBoolean(prefix, Key_UseTransparency, properties, useTransparency);
        useAntialiasing = loadBoolean(prefix, Key_UseAntialiasing, properties, useAntialiasing);
    }

    @Override
    protected void _loadDefaults() {
        randomSeed = "DefaultSeed";
        width = 640;
        height = 480;
        numberOfThreads = 2;
        numberOfIterations = 300;
        numberOfTriangles = 200;
        useTransparency = true;
        useAntialiasing = false;
    }
    
}
