package ch.brotzilla.performance.tests;

import java.awt.image.BufferedImage;

import ch.brotzilla.performance.StaticSettings;
import ch.brotzilla.performance.util.Triangles;

public abstract class PerfTest {

    private final Triangles triangles;
    private final int thread;
    
    private double time = Double.NaN;
    private long pixelSum = -1;

    public PerfTest(Triangles triangles, int thread) {
        this.triangles = triangles;
        this.thread = thread;
    }
    
    public Triangles getTriangles() {
        return triangles;
    }
    
    public int getThread() {
        return thread;
    }
    
    public double getTimeInMS() {
        return time;
    }
    
    public long getPixelSum() {
        return pixelSum;
    }
    
    public abstract BufferedImage getImage();
    
    public void execute() {
        initialize();
        final long start = System.nanoTime();
        for (int i = 0; i < StaticSettings.NumberOfIterations; i++) {
            render();
            pixelSum = computePixelSum();
        }
        terminate();
        final long end = System.nanoTime();
        time = (end - start) / 1000000.0;
    }
    
    protected void initialize() {
        // do nothing by default
    }
    
    protected void terminate() {
        // do nothing by default
    }
    
    protected abstract void render();
    
    protected abstract long computePixelSum();

}
