package ch.brotzilla.performance.tests;

import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;

import ch.brotzilla.performance.StaticSettings;
import ch.brotzilla.performance.util.Triangle;
import ch.brotzilla.performance.util.Triangles;
import ch.brotzilla.performance.util.Utils;

public class PerfTestJava2DBufferedImage extends PerfTest {

    private BufferedImage image;
    private Graphics2D graphics;

    public PerfTestJava2DBufferedImage(Triangles triangles, int thread) {
        super(triangles, thread);
    }

    @Override
    public BufferedImage getImage() {
        return image;
    }
    
    @Override
    protected void initialize() {
        image = new BufferedImage(StaticSettings.Width, StaticSettings.Height, StaticSettings.UseTranslucency ? BufferedImage.TYPE_INT_ARGB : BufferedImage.TYPE_INT_RGB); 
        graphics = (Graphics2D) image.getGraphics();
        if (StaticSettings.UseAntialiasing) {
            graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            graphics.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        }
    }

    @Override
    protected void render() {
    	// NOTE
    	// clearRect() and fillRect() behave differently, depending on translucency, i.e. the buffered image type (TYPE_INT_ARGB vs. TYPE_INT_RGB)
        if (StaticSettings.UseTranslucency) {
            graphics.setBackground(StaticSettings.TranslucentBackgroundColor);
        	graphics.clearRect(0, 0, StaticSettings.Width, StaticSettings.Height);
        } else {
            graphics.setColor(StaticSettings.OpaqueBackgroundColor);
        	graphics.fillRect(0, 0, StaticSettings.Width, StaticSettings.Height);
        }
        for (final Triangle triangle : getTriangles()) {
            graphics.setColor(triangle.getColor());
            graphics.fill(triangle.getShape());
        }
    }

    @Override
    protected long computePixelSum() {
        final int[] pixelData = (int[]) image.getRaster().getDataElements(0, 0, StaticSettings.Width, StaticSettings.Height, new int[StaticSettings.Size]);
        return Utils.computePixelSum(pixelData);
    }

}
