package ch.brotzilla.performance.tests;

import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.RenderingHints;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.awt.image.VolatileImage;

import ch.brotzilla.performance.StaticSettings;
import ch.brotzilla.performance.util.Triangle;
import ch.brotzilla.performance.util.Triangles;
import ch.brotzilla.performance.util.Utils;

public class PerfTestJava2DVolatileImage extends PerfTest {

    private VolatileImage image;
    private Graphics2D graphics;

    private BufferedImage snapshot;

    public PerfTestJava2DVolatileImage(Triangles triangles, int thread) {
        super(triangles, thread);
    }

    @Override
    public BufferedImage getImage() {
        return snapshot;
    }

    @Override
    protected void initialize() {
        image = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration().createCompatibleVolatileImage(StaticSettings.Width, StaticSettings.Height, StaticSettings.UseTranslucency ? Transparency.TRANSLUCENT : Transparency.OPAQUE); 
        graphics = (Graphics2D) image.getGraphics();
        if (StaticSettings.UseAntialiasing) {
            graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            graphics.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        }
    }

    @Override
    protected void render() {
    	// NOTE
    	// clearRect() and fillRect() behave differently, depending on translucency
        if (StaticSettings.UseTranslucency) {
            graphics.setBackground(StaticSettings.TranslucentBackgroundColor);
        	graphics.clearRect(0, 0, StaticSettings.Width, StaticSettings.Height);
        } else {
            graphics.setColor(StaticSettings.OpaqueBackgroundColor);
        	graphics.fillRect(0, 0, StaticSettings.Width, StaticSettings.Height);
        }
        for (final Triangle triangle : getTriangles()) {
            graphics.setColor(triangle.getColor());
            graphics.fill(triangle.getShape());
        }
        snapshot = image.getSnapshot();
    }
    
    @Override
    protected long computePixelSum() {
        final int[] pixelData = (int[]) snapshot.getRaster().getDataElements(0, 0, StaticSettings.Width, StaticSettings.Height, new int[StaticSettings.Size]);
        return Utils.computePixelSum(pixelData);
    }

}
